#include "dialog.h"
#include "ui_dialog.h"
#include <VDream/Log/VLog.h>

Dialog::Dialog(QWidget *parent) :
  QDialog(parent),
  ui(new Ui::Dialog)
{
  ui->setupUi(this);
  this->loadFromDefaultDoc("main/coord");
}

Dialog::~Dialog()
{
  this->saveToDefaultDoc("main/coord");
  delete ui;
}

void Dialog::load(VXML xml)
{
  QRect rect = this->geometry();
  rect.setLeft(xml.getInt("left",     rect.left()));
  rect.setTop(xml.getInt("top",       rect.top()));
  rect.setWidth(xml.getInt("width",   rect.width()));
  rect.setHeight(xml.getInt("height", rect.height()));
  this->setGeometry(rect);

  ui->leGo->setText(xml.getStr("url", ui->leGo->text()));
  ui->leClick->setText(xml.getStr("click", ui->leClick->text()));
}

void Dialog::save(VXML xml)
{
  QRect rect = this->geometry();
  xml.setInt("left",   rect.left());
  xml.setInt("top",    rect.top());
  xml.setInt("width",  rect.width());
  xml.setInt("height", rect.height());

  xml.setStr("url", ui->leGo->text());
  xml.setStr("click", ui->leClick->text());
}

bool Dialog::clickElement(QWebElement* element)
{
  LOG_DEBUG("%p", element);
  bool res;
  QRect rect = element->geometry();
  QMouseEvent press(
    QMouseEvent::MouseButtonPress, rect.center(),
    Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
  res = QApplication::sendEvent(ui->webView, &press);
  if (!res)
  {
    LOG_ERROR("sendEvent(press) return false;");
    return false;
  }

  QMouseEvent release(
    QMouseEvent::MouseButtonRelease, rect.center(),
    Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
  res = QApplication::sendEvent(ui->webView, &release);
  if (!res)
  {
    LOG_ERROR("sendEvent(release) return false;");
    return false;
  }
  return true;
}

void Dialog::on_btnGo_clicked()
{
  ui->webView->load(QUrl(ui->leGo->text()));
}

void Dialog::on_webView_loadProgress(int progress)
{
  ui->progressBar->setValue(progress);
}

void Dialog::on_btnClick_clicked()
{
  QString selectorQuery = ui->leClick->text();
  QWebElementCollection collection = ui->webView->page()->mainFrame()->findAllElements(selectorQuery);
  if (collection.count() == 0)
  {
    QMessageBox::information(this, "error", "can not find for " + selectorQuery);
    return;
  }
  if (collection.count() > 1)
  {
    QString msg = QString::number(collection.count()) + " elements found";
    QMessageBox::information(this, "error", msg);
    return;
  }
  QWebElement element = collection.at(0);
  clickElement(&element);

  QString msg = "<" + element.tagName();

  foreach (QString attributeName, element.attributeNames())
  {
    msg += " " + attributeName + "=\"" + element.attribute(attributeName) + "\"";
  }
  msg += ">";

  QRect rect = element.geometry();
  msg = msg + "\r\n\r\n" +
      "left:"    + QString::number(rect.left()) +
      " top:"    + QString::number(rect.top()) +
      " width:"  + QString::number(rect.width()) +
      " height:" + QString::number(rect.height());
  ui->teClick->setPlainText(msg);
}
